import random 
from simevopy import Environment, Food, Genes, Organism

def setup_base_organism(env: Environment, count=20, start=(0, 0), end=(1000, 1000)):
    if end == (0, 0):
        end = (env.get_width(), env.get_height())

    if end[0] < start[0] or end[1] < start[1]:
        raise ValueError("End point should be greater than start point")

    for _ in range(count):
        x = random.randint(start[0], end[0] - 1)
        y = random.randint(start[1], end[1] - 1)

        dna = chr(40) * 4
        env.add_organism(Organism(Genes(dna)), x, y)

def distribute_food_randomly(env: Environment, food_count=50, start=(0, 0), end=(0, 0)):
    if end == (0, 0):
        end = (env.get_width(), env.get_height())

    if end[0] < start[0] or end[1] < start[1]:
        raise ValueError("End point should be greater than start point")

    for _ in range(food_count):
        x = random.randint(start[0], end[0] - 1)
        y = random.randint(start[1], end[1] - 1)

        env.add_food(Food(), x, y)

def reproduce_organisms(env: Environment):
    organisms = env.get_all_organisms()
    for org in organisms:
        if org.can_reproduce():
            new_org = org.reproduce()
            env.add_organism(new_org, org.get_position()[0], org.get_position()[1])

def remove_all_foods(env: Environment):
    remaining_food = env.get_all_foods()
    for f in remaining_food:
        env.remove_food(f)



import matplotlib.pyplot as plt
import numpy as np
from simevopy import Environment
from utils.common import (
    distribute_food_randomly,
    remove_all_foods,
    reproduce_organisms,
    setup_base_organism,
)
from utils.visualize import visualize_objects

env = Environment(1000, 1000)
setup_base_organism(env, 50)

# 初始化用於存儲每代數據的列表
organism_counts = []
average_sizes = []
average_speeds = []
average_awareness = []

for i in range(2000):
    print("=========================================")
    print(f"Gen {i} th")

    distribute_food_randomly(env, 50)

    env.simulate_iteration(
        100,
    )

    organisms = env.get_all_organisms()
    organism_count = len(organisms)
    organism_counts.append(organism_count)

    if organism_count > 0:
        sizes = np.array([org.get_size() for org in organisms])
        speeds = np.array([org.get_speed() for org in organisms])
        awareness = np.array([org.get_awareness() for org in organisms])

        average_sizes.append(np.mean(sizes))
        average_speeds.append(np.mean(speeds))
        average_awareness.append(np.mean(awareness))
    else:
        average_sizes.append(0)
        average_speeds.append(0)
        average_awareness.append(0)

    print("After iteration, organism count: ", organism_count)

    if i % 100 == 0:
        visualize_objects(env, f"Organisms and Food Distribution (Generation {i})")
        input("Press Enter to continue...")

    reproduce_organisms(env)
    remove_all_foods(env)

# 繪製每代的結果圖表
plt.figure(figsize=(10, 8))
plt.subplot(4, 1, 1)
plt.plot(organism_counts, label="Organism Count")
plt.legend()

plt.subplot(4, 1, 2)
plt.plot(average_sizes, label="Average Size")
plt.legend()

plt.subplot(4, 1, 3)
plt.plot(average_speeds, label="Average Speed")
plt.legend()

plt.subplot(4, 1, 4)
plt.plot(average_awareness, label="Awareness Radius")
plt.legend()

plt.xlabel("Generation")
plt.show()



from simevopy import Environment
from utils.common import (
    distribute_food_randomly,
    remove_all_foods,
    reproduce_organisms,
    setup_base_organism,
)
from utils.visualize import visualize_objects

env = Environment(1000, 1000)
setup_base_organism(env, 50)

for i in range(100):
    print("=========================================")
    print(f"Gen {i} th")

    distribute_food_randomly(env, 50)

    # visualize_objects(env, f"Organisms and Food Distribution (Generation {i})")
    env.simulate_iteration(
        100,
        on_each_iteration=lambda env: visualize_objects(
            env, f"Organisms and Food Distribution (Generation {i})"
        ),
    )

    print("After iteration, organism count: ", len(env.get_all_organisms()))

    reproduce_organisms(env)
    remove_all_foods(env)

    # input("Press Enter to continue...")



import math

from simevopy import Environment, Food, Genes, Organism
from utils.visualize import visualize_objects

def distribute_food_in_circle(env, food_count=30):
    for i in range(food_count):
        angle = i * 2 * math.pi / food_count
        x = 500 + math.cos(angle) * 300
        y = 500 + math.sin(angle) * 300
        env.add_food(Food(), x, y)


env = Environment(1000, 1000)
dna = chr(40) * 2 + chr(120) * 2

env.add_organism(Organism(Genes(dna)), 275, 275)
distribute_food_in_circle(env, food_count=80)
visualize_objects(env)
env.simulate_iteration(300, on_each_iteration=visualize_objects)

input("Press Enter to continue...")
